<?php

/**
 * Checks if the given username already exists.
 */
function auth_usernameExists(string $username): bool
{
	return db_exists(db_auth(), 'users', 'username', $username);
}

/**
 * Checks if the given email already exists.
 */
function auth_emailExists(string $email): bool
{
	return db_exists(db_auth(), 'users', 'email', $email);
}

/**
 * Displays the registration page.
 */
function auth_register_get(): void
{
	echo render('layouts/basic', ['view' => 'pages/auth/register']);
}

/**
 * Handles the registration form submission.
 */
function auth_register_post(): void
{
	csrf_ensure();

	$errors = [];

	$u = $_POST['username'] ?? '';
	$e = $_POST['email'] ?? '';
	$p = $_POST['password'] ?? '';

	// Trim the input.
	$u = trim($u);
	$e = trim($e);

	/*
		A username is required.
		A username must be at least 3 characters long and at most 25 characters long.
		A username must contain only alphanumeric characters and spaces.
	*/
	if (empty($u) || strlen($u) < 3 || strlen($u) > 25 || !ctype_alnum(str_replace(' ', '', $u))) {
		$errors['u'][] = 'Username is required and must be between 3 and 25 characters long and contain only
			alphanumeric characters and spaces.';
	}

	/*
		An email is required.
		An email must be at most 255 characters long.
		An email must be a valid email address.
	*/
	if (empty($e) || strlen($e) > 255 || !filter_var($e, FILTER_VALIDATE_EMAIL)) {
		$errors['e'][] = 'Email is required must be a valid email address.';
	}

	/*
		A password is required.
		A password must be at least 6 characters long.
	*/
	if (empty($p) || strlen($p) < 6) {
		$errors['p'][] = 'Password is required and must be at least 6 characters long.';
	}

	// If there are errors at this point, send them to the page with errors flashed.
	if (!empty($errors)) {
		flash('errors', $errors);
		redirect('/auth/register');
	}

	/*
		A username must be unique.
	*/
	if (auth_usernameExists($u)) {
		$errors['u'][] = 'Username is already taken.';
	}

	/*
		An email must be unique.
	*/
	if (auth_emailExists($e)) {
		$errors['e'][] = 'Email is already taken.';
	}

	// If there are errors at this point, send them to the page with errors flashed.
	if (!empty($errors)) {
		flash('errors', $errors);
		redirect('/auth/register');
	}

	$user = user_create($u, $e, $p);
	if ($user === false) router_error(400);

	$_SESSION['user'] = user_find($u);
	redirect('/');
}

/**
 * Displays the login page.
 */
function auth_login_get(): void
{
	echo render('layouts/basic', ['view' => 'pages/auth/login']);
}

/**
 * Handles the login form submission.
 */
function auth_login_post(): void
{
	csrf_ensure();

	$errors = [];

	$u = $_POST['username'] ?? '';
	$p = $_POST['password'] ?? '';

	// Trim the input.
	$u = trim($u);

	/*
		A username is required.
	*/
	if (empty($u)) {
		$errors['u'][] = 'Username is required.';
	}

	/*
		A password is required.
	*/
	if (empty($p)) {
		$errors['p'][] = 'Password is required.';
	}

	// If there are errors at this point, send them to the page with errors flashed.
	if (!empty($errors)) {
		flash('errors', $errors);
		redirect('/auth/login');
	}

	$user = user_find($u);
	if ($user === false || !password_verify($p, $user['password'])) {
		$errors['u'][] = 'Invalid username or password.';
		flash('errors', $errors);
		redirect('/auth/login');
	}

	$_SESSION['user'] = $user;
	if ($_POST['remember'] ?? false) auth_rememberMe();
	redirect('/');
}

/**
 * Logs the user out.
 */
function auth_logout(): void
{
	csrf_ensure();
	session_delete($_SESSION['user']['id']);
	unset($_SESSION['user']);
	set_cookie('remember_me', '', 1);
	redirect('/');
}

/**
 * Create a long-lived session for the user.
 */
function auth_rememberMe()
{
	$token = token();
	$expires = strtotime('+30 days');
	$result = db_query(db_auth(), "INSERT INTO sessions (token, user_id, expires) VALUES (:t, :u, :e)", [
		':t' => $token,
		':u' => $_SESSION['user']['id'],
		':e' => $expires
	]);
	if (!$result) router_error(400);
	set_cookie('remember_me', $token, $expires);
}

/**
 * Check for a user session. If $_SESSION['user'] already exists, return early. If not, check for a remember me
 * cookie. If a remember me cookie exists, validate the session and set $_SESSION['user'].
 */
function auth_check(): bool
{
	if (isset($_SESSION['user'])) return true;

	if (isset($_COOKIE['remember_me'])) {
		$session = session_validate($_COOKIE['remember_me']);
		if ($session === true) {
			$user = user_find($session['user_id']);
			unset($user['password']);
			$_SESSION['user'] = user_find($session['user_id']);
			return true;
		}
	}

	return false;
}
